
local _,modmod = ...

ModifiedModifier_Global = {} -- indexed by binding command, keys bound to commands
ModifiedModifier_PerCharacter = {}

-- this is the binding command
modmod.TheKey = "CLICK ModifiedModifier:LeftButton"

BINDING_HEADER_MODIFIEDMODIFIER = "Modified Modifier"
_G["BINDING_NAME_"..modmod.TheKey] = "ModMod Key"

modmod.button = CreateFrame("Button","ModifiedModifier",nil,"SecureHandlerClickTemplate")
modmod.button:RegisterForClicks("AnyUp","AnyDown")
modmod.button:SetAttribute("_onclick",[[
	if down then -- if key is down
		for i=1,self:GetAttribute("numBindings") do -- set all defined override bindings
			self:SetBinding(false,self:GetAttribute("bindingKey"..i),self:GetAttribute("bindingCmd"..i))
		end
	else -- if key is up
		self:ClearBindings() -- clear all override bindings
	end
]])

--[[ UI ]]

modmod.ui = CreateFrame("Frame","ModModUI")

function modmod.ui:ADDON_LOADED(addon)
	if addon=="Blizzard_BindingUI" then
		self:AdjustKeyBindingFrame()
	end
end

function modmod.ui:PLAYER_LOGIN()
	modmod.ui:PopulateButtonAttributes()
end

modmod.ui:SetScript("OnEvent",function(self,event,...)
	if self[event] then
		self[event](self,...)
	end
end)
modmod.ui:RegisterEvent("ADDON_LOADED")
modmod.ui:RegisterEvent("PLAYER_LOGIN")

-- to pass a table we define outside the secure environment into a secure
-- snippet, define attributes bindingKey1, bindingCmd1, bindingKey2, etc
-- with numBindings attribute being the total bindings
function modmod.ui:PopulateButtonAttributes()
	if InCombatLockdown() then return end
	-- wipe old attributes
	local button = modmod.button
	local saved = GetCurrentBindingSet()==1 and ModifiedModifier_Global or ModifiedModifier_PerCharacter
	local index = 0
	for cmd,key in pairs(saved) do
		local TheKey1,TheKey2 = GetBindingKey(modmod.TheKey)
		if key~=TheKey1 and key~=TheKey2 then
			-- add only if key is not TheKey (or its alternate if one bound)
			index = index + 1
			button:SetAttribute("bindingKey"..index,key)
			button:SetAttribute("bindingCmd"..index,cmd)
		end
	end
	button:SetAttribute("numBindings",index)
end

local function nudge(widget,xoff,yoff)
	for i=1,widget:GetNumPoints() do
		local anchorPoint, relativeTo, relativePoint, x, y = widget:GetPoint(i)
		widget:SetPoint(anchorPoint, relativeTo, relativePoint, x+xoff, y+yoff)
	end
end

-- adds a column of modmod binding buttons to the default key binding interface
function modmod.ui:AdjustKeyBindingFrame()
	local frame = KeyBindingFrame

	nudge(KeyBindingFrameKeyBinding1,-24,0)

	self.bindings = {}
	for i=1,21 do
		local key1 = _G["KeyBindingFrameKeyBinding"..i.."Key1Button"]
		key1:SetWidth(160)
		nudge(key1,-16,0)
		local key2 = _G["KeyBindingFrameKeyBinding"..i.."Key2Button"]
		key2:SetWidth(160)
		self.bindings[i] = CreateFrame("Button",nil,key2:GetParent(),"UIMenuButtonStretchTemplate")
		self.bindings[i]:SetSize(92,22)
		self.bindings[i]:SetPoint("LEFT",key2,"RIGHT")
		self.bindings[i]:SetText("Test")
		self.bindings[i]:RegisterForClicks("AnyUp")
		self.bindings[i]:SetScript("OnClick",self.Binding_OnClick)
	end

	nudge(frame.commandLabel,-24,0)
	nudge(frame.key1Label,-26,0)
	nudge(frame.key2Label,-26,0)

	frame.modModLabel = frame:CreateFontString(nil,"BORDER","GameFontWhite")
	frame.modModLabel:SetText("ModMod")
	frame.modModLabel:SetPoint("LEFT",frame.key2Label,"RIGHT",84,0)

	self.unbind = CreateFrame("Button",nil,frame,"UIPanelButtonTemplate")
	self.unbind:SetSize(130,22)
	self.unbind:SetAllPoints(KeyBindingFrame.unbindButton)
	self.unbind:SetText("Unbind ModMod")
	self.unbind:Hide()
	self.unbind:SetScript("OnClick",function(self)
		local saved = GetCurrentBindingSet()==1 and ModifiedModifier_Global or ModifiedModifier_PerCharacter
		if modmod.selected and saved[modmod.selected] then
			saved[modmod.selected] = nil
		end
		KeyBindingFrame.outputText:SetText("")
		modmod.selected = nil
		KeyBindingFrame_SetSelected(nil)
		modmod.ui:KeyBindingFrame_Update()
		modmod.ui:PopulateButtonAttributes()
		modmod.ui:UpdateUnbind()
	end)

	hooksecurefunc("KeyBindingFrame_Update",self.KeyBindingFrame_Update)
	hooksecurefunc("KeyBindingFrame_SetSelected",self.KeyBindingFrame_SetSelected)
	-- OnKeyDown hooks
	KeyBindingFrame:HookScript("OnKeyDown",self.KeyBindingFrame_OnKeyDown) -- keyboard
	KeyBindingFrame:HookScript("OnClick",function(self,button) -- mouse button
		modmod.ui:KeyBindingFrame_OnKeyDown(button=="LeftButton" and "BUTTON1" or
			button=="RightButton" and "BUTTON2" or button=="MiddleButton" and "BUTTON3" or button:upper())
	end)

	-- one for scrollframe (for scrollable lists)
	KeyBindingFrameScrollFrame:HookScript("OnMouseWheel",function(self,delta) -- mousewheel
		modmod.ui:KeyBindingFrame_OnKeyDown(delta>0 and "MOUSEWHEELUP" or "MOUSEWHEELDOWN")
	end)
	-- and another for parent frame (when scrollframe not active)
	KeyBindingFrame:HookScript("OnMouseWheel",function(self,delta)
		modmod.ui:KeyBindingFrame_OnKeyDown(delta>0 and "MOUSEWHEELUP" or "MOUSEWHEELDOWN")
	end)

end

function modmod.ui:Binding_OnClick(button)
	if modmod.selected and modmod.selected==self.command and button~="LeftButton" then
		modmod.ui:KeyBindingFrame_OnKeyDown(button=="RightButton" and "BUTTON2" or button=="MiddleButton" and "BUTTON3" or button:upper())
		return
	end

	local oldselected = modmod.selected
	KeyBindingFrame_SetSelected(nil)
	KeyBindingFrame.outputText:SetText("")
	KeyBindingFrame_Update()
	if modmod.selected and modmod.selected==self.command then
		modmod.selected = nil
		self:UnlockHighlight()
	else
		modmod.selected = self.command
		self:LockHighlight()
		KeyBindingFrame.outputText:SetText(format("Press Key to Bind with ModMod for -> %s",_G["BINDING_NAME_"..self.command] or self.command))
	end
	modmod.fauxOffset = FauxScrollFrame_GetOffset(KeyBindingFrameScrollFrame)
	modmod.ui.UpdateUnbind()

end

function modmod.ui:UpdateUnbind()
	if modmod.selected then
		KeyBindingFrame.unbindButton:Hide()
		modmod.ui.unbind:Show()
		local saved = GetCurrentBindingSet()==1 and ModifiedModifier_Global or ModifiedModifier_PerCharacter
		if saved[modmod.selected] then
			modmod.ui.unbind:Enable()
		else
			modmod.ui.unbind:Disable()
		end
	else
		KeyBindingFrame.unbindButton:Show()
		modmod.ui.unbind:Hide()
	end
end

function modmod.ui:KeyBindingFrame_Update()
	local bindings = modmod.ui.bindings
	local saved = GetCurrentBindingSet()==1 and ModifiedModifier_Global or ModifiedModifier_PerCharacter
	for i=1,21 do
		local key1 = _G["KeyBindingFrameKeyBinding"..i.."Key1Button"]
		local button = bindings[i]
		button:UnlockHighlight()
		if key1:IsShown() then
			local command = key1.commandName
			button.command = command
			if saved[command] then
				local binding = GetBindingText(saved[command],"KEY_"):gsub("Mouse Wheel","Wheel") -- shorten text for narrower buttons
				binding = binding=="Left Mouse Button" and "Left Mouse" or binding=="Right Mouse Button" and "Right Mouse" or binding
				button:SetText(binding)
				button:SetAlpha(1.0)
			else
				button:SetText(GRAY_FONT_COLOR_CODE..NOT_BOUND..FONT_COLOR_CODE_CLOSE)
				button:SetAlpha(0.8)
			end
			button:Show()
		else
			button:Hide()
		end
	end
end

function modmod.ui:KeyBindingFrame_OnKeyDown(key)
	-- don't do anything if: no modmod binding selected, PrintScreen, ESC or an unknown key is hit,
	-- and if the defined modmod key is pressed
	if not modmod.selected or key=="PRINTSCREEN" or key=="ESCAPE" or key=="UNKNOWN" or key==GetBindingKey(modmod.TheKey) then
		return
	end
	if IsModifierKeyDown() then
		KeyBindingFrame.outputText:SetText("|cFFFF0000Sorry! Alt, Ctrl and Shift are not supported for ModMod key combos.")
		return
	end
	local saved = GetCurrentBindingSet()==1 and ModifiedModifier_Global or ModifiedModifier_PerCharacter
	local already = saved[modmod.selected] == key -- same key is already bound to same command
	local unbound
	if not already then -- if a new key for this command
		for c,k in pairs(saved) do -- look for any of the same key that need unbound
			if k==key then
				unbound = c
				saved[c] = nil
			end
		end
		if unbound then
			KeyBindingFrame.outputText:SetText("|cFFFF0000ModMod "..(_G["BINDING_NAME_"..unbound] or unbound).." function is Now Unbound!")
		else
			KeyBindingFrame.outputText:SetText("ModMod Key Bound Successfully")
		end
	else
		KeyBindingFrame.outputText:SetText("")
	end
	saved[modmod.selected] = key
	modmod.selected = nil
	modmod.ui.KeyBindingFrame_SetSelected(nil)
	modmod.ui:KeyBindingFrame_Update()
	modmod.ui.UpdateUnbind()
	if key=="MOUSEWHEELDOWN" or key=="MOUSEWHEELUP" then
		-- since this is a post-hook on a mousewheel, the verticalscroll has already happened
		-- we noted the offset when a binding button was hit. return to that offset to pretend nothing happened
		FauxScrollFrame_OnVerticalScroll(KeyBindingFrameScrollFrame, modmod.fauxOffset*KEY_BINDING_HEIGHT, KEY_BINDING_HEIGHT, KeyBindingFrame_Update)
	end
	modmod.ui:PopulateButtonAttributes()
end

function modmod.ui.KeyBindingFrame_SetSelected(command)
	if command then
		modmod.selected = nil
	end
	for i=1,17 do
		modmod.ui.bindings[i]:UnlockHighlight()
	end
end
